# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import

import json
import time

from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import (
    Temp,
    Fan,
    PSU)
from netq_lib.common.enums import ServiceStatus


def process_sensors(output, svc_failed=False, **kwargs):
    failed = False
    retcode = ServiceStatus.OK

    if svc_failed:
        failed = True
        retcode = ServiceStatus.FAIL
    else:
        try:
            s_list = json.loads(output)
        except ValueError:
            failed = True
            retcode = ServiceStatus.FAIL

    if failed:
        tm_stamp = time.time()
        msg = 'Sensor Read Failure'
        state = 'unknown'
        for fan in Fan.query.filter(hostname=HOSTNAME):
            fan.s_state = state.lower()
            fan.s_input = 0
            fan.s_msg = msg
            fan.timestamp = tm_stamp
            fan.save()
        for psu in PSU.query.filter(hostname=HOSTNAME):
            psu.s_state = state.lower()
            psu.timestamp = tm_stamp
            psu.s_msg = msg
            psu.save()
        for temp in Temp.query.filter(hostname=HOSTNAME):
            temp.s_state = state.lower()
            temp.s_input = 0.0
            temp.timestamp = tm_stamp
            temp.s_msg = msg
            temp.save()
        return retcode

    for s in s_list:
        sensor = None
        if s['type'] == 'temp':
            sensor = Temp()
            if 'input' in s:
                sensor.s_input = s['input']
                sensor.s_crit = s['crit']
                sensor.s_max = s['max']
                sensor.s_min = s['min']
                sensor.s_lcrit = s['lcrit']
            else:
                sensor.s_input = 0.0
                sensor.s_max = 0
                sensor.s_min = 0
                sensor.s_crit = 0
                sensor.s_lcrit = 0
            sensor.s_desc = s['description'].encode('ascii', 'ignore').lower()
        elif s['type'] == 'fan':
            sensor = Fan()
            if 'input' in s:
                sensor.s_input = s['input']
                sensor.s_max = s['max']
                sensor.s_min = s['min']
            else:
                sensor.s_input = 0
                sensor.s_max = 0
                sensor.s_min = 0
            sensor.s_desc = s['description'].encode('ascii', 'ignore').lower()

        elif s['type'] == 'power':
            sensor = PSU()

        if sensor is not None:
            sensor.s_name = s['name'].encode('ascii', 'ignore').lower()
            if not s['msg']:
                sensor.s_msg = ''
            else:
                msg = s['msg'].encode('ascii', 'ignore')

                msgs = msg.split('\n')
                m = []
                for msg in msgs:
                    m.append(msg.split(':', 1)[1])
                sensor.s_msg = ','.join(m)
            sensor.s_state = s['state'].encode('ascii', 'ignore').lower()
            sensor.s_prev_state = s['prev_state'].encode('ascii',
                                                         'ignore').lower()
            sensor.save()
    return retcode
